@@ -834,7 +834,7 @@ su_ask_password (GksuContext *context, gchar *prompt,
     dialog = gksuui_dialog_new (context->sudo_mode);
 
   if (prompt)
-    gksuui_dialog_set_prompt (GKSUUI_DIALOG(dialog), prompt);
+    gksuui_dialog_set_prompt (GKSUUI_DIALOG(dialog), _(prompt));
 
   if (context->message)
     gksuui_dialog_set_message (GKSUUI_DIALOG(dialog), context->message);
@@ -1759,6 +1759,7 @@ gksu_su_full (GksuContext *context,
     {
       gchar **cmd = g_malloc (sizeof(gchar*)*7);
 
+      setsid();   // make us session leader
       cmd[i] = g_strdup ("/bin/su"); i++;
       if (context->login_shell)
 	{
@@ -1898,93 +1899,120 @@ gksu_su_full (GksuContext *context,
 	    fprintf (stderr, "DEBUG (run:post-after-pass) buf: -%s-\n", buf);
 	}
 
-      if (!strncmp (buf, "su", 2))
+      FD_ZERO (&rfds);
+      FD_SET (fdpty, &rfds);
+      tv.tv_sec = 1;
+      tv.tv_usec = 0;
+      int loop_count = 0;
+      while (TRUE)
 	{
-	  gchar **strings;
+	  int retval = 0;
 
-	  if (password)
+	  if (!strncmp (buf, "su", 2))
 	    {
-	      nullify_password (password);
-	      unset_gnome_keyring_password (context);
+	      gchar **strings;
+
+	      if (password)
+		{
+		  nullify_password (password);
+		  unset_gnome_keyring_password (context);
+		}
+
+	      strings = g_strsplit (buf, ":", 2);
+	      if (strings[1] && !strncmp (strings[1], " Authentication failure", 23))
+		g_set_error (error, gksu_quark,
+			     GKSU_ERROR_WRONGPASS,
+			     _("Wrong password."));
+	      g_strfreev (strings);
+
+	      if (context->debug)
+		fprintf (stderr, "DEBUG (auth_failed) buf: -%s-\n", buf);
+
+	      break;
+	    }
+	  else if (!strncmp (buf, "gksu: waiting", 13))
+	    {
+	      gchar *line;
+
+	      if (password)
+		{
+		  set_gnome_keyring_password (context, password);
+		  nullify_password (password);
+		}
+
+	      if (context->debug)
+		fprintf (stderr, "DEBUG (gksu: waiting) buf: -%s-\n", buf);
+
+	      line = g_strdup_printf ("gksu-run: %s\n", context->display);
+	      write (fdpty, line, strlen(line));
+	      g_free (line);
+
+	      line = g_strdup_printf ("gksu-run: %s\n", context->xauth);
+	      write (fdpty, line, strlen(line));
+	      g_free (line);
+
+	      tcdrain (fdpty);
+
+	      bzero (buf, 256);
+	      read (fdpty, buf, 255);
+
+	      break;
 	    }
 
-	  strings = g_strsplit (buf, ":", 2);
-	  if (strings[1] && !strncmp (strings[1], " Authentication failure", 23))
-	    g_set_error (error, gksu_quark,
-			 GKSU_ERROR_WRONGPASS,
-			 _("Wrong password."));
-	  g_strfreev (strings);
-
-	  if (context->debug)
-	    fprintf (stderr, "DEBUG (auth_failed) buf: -%s-\n", buf);
-	}
-      else if (!strncmp (buf, "gksu: waiting", 13))
-	{
-	  gchar *line;
-
-	  if (password)
+	  retval = select (fdpty + 1, &rfds, NULL, NULL, &tv);
+	  if ((loop_count > 50) || (!retval))
 	    {
-	      set_gnome_keyring_password (context, password);
-	      nullify_password (password);
+	      gchar *emsg = NULL;
+	      gchar *converted_str = NULL;
+	      GError *converr = NULL;
+
+	      if (password)
+		nullify_password (password);
+
+	      converted_str = g_locale_to_utf8 (buf, -1, NULL, NULL, &converr);
+	      if (converr)
+		{
+		  g_warning (_("Failed to communicate with "
+			       "gksu-run-helper.\n\n"
+			       "Received:\n"
+			       " %s\n"
+			       "While expecting:\n"
+			       " %s"), buf, "gksu: waiting");
+		  emsg = g_strdup_printf (_("Failed to communicate with "
+					    "gksu-run-helper.\n\n"
+					    "Received bad string "
+					    "while expecting:\n"
+					    " %s"), "gksu: waiting");
+		  g_error_free (converr);
+		}
+	      else
+		emsg = g_strdup_printf (_("Failed to communicate with "
+					  "gksu-run-helper.\n\n"
+					  "Received:\n"
+					  " %s\n"
+					  "While expecting:\n"
+					  " %s"), converted_str, "gksu: waiting");
+	      g_free (converted_str);
+
+	      g_set_error (error, gksu_quark, GKSU_ERROR_HELPER, emsg);
+	      g_free (emsg);
+
+	      if (context->debug)
+		fprintf (stderr, "DEBUG (failed!) buf: -%s-\n", buf);
+
+	      return FALSE;
+	    }
+	  else if (retval == -1)
+	    {
+	      if (context->debug)
+		fprintf (stderr, "DEBUG (select failed!) buf: %s\n", buf);
+	      return FALSE;
 	    }
 
-	  if (context->debug)
-	    fprintf (stderr, "DEBUG (gksu: waiting) buf: -%s-\n", buf);
-
-	  line = g_strdup_printf ("gksu-run: %s\n", context->display);
-	  write (fdpty, line, strlen(line));
-	  g_free (line);
-
-	  line = g_strdup_printf ("gksu-run: %s\n", context->xauth);
-	  write (fdpty, line, strlen(line));
-	  g_free (line);
-
-	  tcdrain (fdpty);
-
-	  bzero (buf, 256);
 	  read (fdpty, buf, 255);
-	}
-      else
-	{
-	  gchar *emsg = NULL;
-	  gchar *converted_str = NULL;
-	  GError *converr = NULL;
-
-	  if (password)
-	    nullify_password (password);
-
-	  converted_str = g_locale_to_utf8 (buf, -1, NULL, NULL, &converr);
-	  if (converr)
-	    {
-	      g_warning (_("Failed to communicate with "
-			   "gksu-run-helper.\n\n"
-			   "Received:\n"
-			   " %s\n"
-			   "While expecting:\n"
-			   " %s"), buf, "gksu: waiting");
-	      emsg = g_strdup_printf (_("Failed to communicate with "
-					"gksu-run-helper.\n\n"
-					"Received bad string "
-					"while expecting:\n"
-					" %s"), "gksu: waiting");
-	      g_error_free (converr);
-	    }
-	  else
-	    emsg = g_strdup_printf (_("Failed to communicate with "
-				      "gksu-run-helper.\n\n"
-				      "Received:\n"
-				      " %s\n"
-				      "While expecting:\n"
-				      " %s"), converted_str, "gksu: waiting");
-	  g_free (converted_str);
-
-	  g_set_error (error, gksu_quark, GKSU_ERROR_HELPER, emsg);
-	  g_free (emsg);
-
 	  if (context->debug)
-	    fprintf (stderr, "DEBUG (failed!) buf: -%s-\n", buf);
-
-	  return FALSE;
+	    fprintf (stderr, "DEBUG (run:after-pass) buf: -%s-\n", buf);
+	  loop_count++;
 	}
 
       if (!password_needed || used_gnome_keyring)
@@ -2435,11 +2463,15 @@ gksu_sudo_full (GksuContext *context,
 
 	  nullify_password (password);
 
-	  /* ignore the newline that comes right after sudo receives
-	     the password
-	  */
+	  /* turn NONBLOCK off */
+	  fcntl(parent_pipe[0], F_SETFL, fcntl(parent_pipe[0], F_GETFL) & ~O_NONBLOCK);
+	  /* ignore the first newline that comes right after sudo receives
+	     the password */
 	  fgets (buffer, 255, infile);
+	  /* this is the status we are interessted in */
 	  fgets (buffer, 255, infile);
+	  /* turn it ON again */
+	  fcntl(parent_pipe[0], F_SETFL, fcntl(parent_pipe[0], F_GETFL) | O_NONBLOCK);
 	}
       else
 	{
@@ -2676,7 +2708,7 @@ gchar*
 gksu_ask_password_full (GksuContext *context, gchar *prompt,
 			GError **error)
 {
-  gchar *ret_value = su_ask_password (context, prompt, NULL, error);
+  gchar *ret_value = su_ask_password (context, _(prompt), NULL, error);
   if (context->sn_context)
     gksu_context_launch_complete (context);
   return ret_value;
